<?php

/**
*
* Trackback Resource Class for MODx 0.9.5
*
*  Original author : Ran Aroussi <ran@blogish.org> 
*  2nd      author : Florian Leifeld (info@florian-leifeld.de)
*
*  Author  : soushi
*  Version : 1.03
*  Date    : 2007/5/20
*
*  License : GPL
*
*/

class Trackback{

  var $author = '';              // Default author name used throughout the class.
  var $forceAuthor = false;      // All Document set $author
  var $tblTrackback = '';        // Trackback Data table
  var $excerptContentSize = 255; // excerpt string length(except introtext)
  var $StringSize = 255;         // URL,blog_name,title string length
  var $mbEnable = false;         // multibyte string function support
  var $allowParentID = array();  // Trackback allow parent ID
  var $allowDocID = array();     // Trackback allow document ID
  var $encoding = ''; // Default encoding used throughout the class (ie. UTF-8)
  var $DEBUG = false;             // DEBUG MODE
  var $VERSION = '1.10';          // This resource version

  //Not use 'const' on php4....
  var $ckAllow  = '0';
  var $ckDeny   = '1';
  var $ckExpire = '2';

  //-----------------------------------------------------
  // Class Constructure
  // 
  // $encoding : string : text encode
  // $pid      : string : Trackback allow parent IDs
  // return : none
  //-----------------------------------------------------
  function Trackback($encoding = "UTF-8",$pid='',$debug=false){
    global $modx;

    $this->encoding     = $encoding;
    $this->tblTrackback = $modx->db->config['table_prefix'] . 'site_trackback';
    $this->DEBUG = $debug == true ? true : false;

    $tmppid =  $pid == '' ? '0+' : $pid;
    $tmppid =  explode(',',$tmppid);
    foreach($tmppid as $key => $val){
      if(! preg_match('/^([0-9]+)/',$val,$match)){
	$this->Pnt('allowParentID not found:'.$val);
      }else{
	$tmpid = $match[1];
	if( strpos($val,'+') === false){
	  $this->allowParentID[$tmpid][0] = false;
	}else{
	  $this->allowParentID[$tmpid][0] = true;
	}
      if( preg_match('/c([0-9]+)/i',$val,$match)){
	  $this->allowParentID[$tmpid][1] = $match[1] * 60 * 60 * 24;
	  $this->Pnt('match'.$val);
	}else{
	  $this->allowParentID[$tmpid][1] = 0;
	  $this->Pnt('no match'.$val);
	}
      }
    }
    
    //$this->allowParentID = $pid == '' ? '0+' : explode(',',$pid);


    if( ! is_array($this->allowParentID)){
      $this->allowParentID = array($this->allowParentID);
    }

    if(extension_loaded("mbstring")){
      $this->mbEnable = true;
    }

  } 

  //-----------------------------------------------------
  //  Count Trackbacks
  //
  //  return : integer : trackback count(blank is error)
  //-----------------------------------------------------
  function TrackbackCount($parent_id = 0){
    global $modx;

    if( $this->CheckAllowTrackback($parent_id) == $this->ckDeny){
      return '';
    }

    if(! $rs = $modx->db->select("id",
				 $this->tblTrackback,
				 "     parent_id = ".$parent_id.
				 " and view = 1")){
      $this->Pnt('DB Query failed.','TrackbackCount');
      return '';
    }

    $cnt=0;
    while ($row= $modx->db->getRow($rs)) {
      $cnt++;
    }

    return $cnt;

  } 

  //-----------------------------------------------------
  //  Make RDF(auto-discovery)
  //
  //  $docid : integer : document id
  //  $recid : integer : trackback receive document id
  //  return : strings :
  //-----------------------------------------------------
  function MakeRDF($docid = 0,$recid){
    global $modx,$_lang;

    if( !is_numeric($recid) || $recid < 1 ){
      return '<!-- '.$_lang['no_receive_id'].'-->';
    }

    $rslt = $this->CheckAllowTrackback($docid);
    switch ($rslt) {
    case $this->ckDeny:
      return '<!-- '.$_lang['deny_trackback'].'-->';
      break;
    case $this->ckExpire:
      return '<!-- '.$_lang['expire_trackback'].'-->';
      break;
    }

    $docObj = $modx->getDocumentObject('id',$docid);

    $rdf_title   = $docObj['pagetitle'];
    if( $docObj['introtext'] != "" ){
      $rdf_excerpt = $modx->stripTags($docObj['introtext']);
    }else{
      $rdf_excerpt = $this->CutText($modx->stripTags($docObj['content']),$this->excerptContentSize,true);
    }

    $rdf_date = $this->TimeToW3CDTF($docObj['createdon']);
    $rdf_link = $modx->makeUrl($docid,'','','full');
    $rdf_ping = $modx->makeUrl($recid,'','tb_id=' . $docid,'full');

    $rdf_author = $this->GetAuthor($docObj['createdby']);

    $output = "<!-- \n"
      . "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" \n"
      . "	xmlns:dc=\"http://purl.org/dc/elements/1.1/\" \n"
      . "	xmlns:trackback=\"http://madskills.com/public/xml/rss/module/trackback/\"> \n"
      . "<rdf:Description \n"
      . "	rdf:about=\"" . $this->xml_safe($rdf_link) . "\" \n"
      . "	dc:identifier=\"" . $this->xml_safe($rdf_link) . "\" \n"
      . "	trackback:ping=\"" . $this->xml_safe($rdf_ping) . "\" \n"
      . "	dc:title=\"" . $this->xml_safe($rdf_title) . "\" \n"
      . "	dc:subject=\"TrackBack\" \n"
      . "	dc:description=\"" . $this->xml_safe($rdf_excerpt) . "\" \n"
      . "	dc:creator=\"" . $this->xml_safe($rdf_author) . "\" \n"
      . "	dc:date=\"" . $rdf_date . "\" /> \n"
      . "</rdf:RDF> \n"
      . "-->  \n";

    return $output;

  } 

  //-----------------------------------------------------
  //  Make trackback URL
  //
  //  $docid     : integer : document id
  //  $recid     : integer : receive document id
  //  return     : strings : 
  //-----------------------------------------------------
  function MakeURL($docid = 0,$recid){
    global $modx,$_lang;

    if( !is_numeric($recid) || $recid < 1 ){
      return $_lang['no_receive_id'];
    }

    $rslt = $this->CheckAllowTrackback($docid);
    switch ($rslt) {
    case $this->ckDeny:
      return $_lang['deny_trackback'];
      break;
    case $this->ckExpire:
      return $_lang['expire_trackback'];
      break;
    }

    return $modx->makeUrl($recid,'','tb_id=' . $docid,'full');

  }

  //-----------------------------------------------------
  //  Make tackback HTML
  //
  //  $docid     : integer : document id
  //  $recid     : integer : receive document id
  //  $tbTpl     : strings : Trackback template
  //  $tbListTpl : strings : Trackback template(list)
  //  return     : strings : HTML code
  //-----------------------------------------------------
  function MakeTrackback($docid = 0,$recid,$tbTpl,$tbListTpl){
    global $modx,$_lang;

    if( !is_numeric($docid) || $docid < 0 ){
      return $_lang['invalid_docid'];
    }elseif( $this->CheckAllowTrackback($docid) == $this->ckDeny){
      return $_lang['deny_trackback'];
    }

    $output = $tbTpl;
    if(! (strpos($tbTpl,'[+tburl+]') === false) ){
      $output = str_replace('[+tburl+]',$this->MakeURL($docid,$recid),$output);
    }
    
    if(! (strpos($tbTpl,'[+tbrdf+]') === false) ){
      $output = str_replace('[+tbrdf+]',$this->MakeRDF($docid,$recid),$output);
    }

    if(! (strpos($tbTpl,'[+tblist+]') === false) ){
      $output = str_replace('[+tblist+]',$this->MakeTrackbackList($docid,$tbListTpl),$output);
    }

    return $output;


  }

  //-----------------------------------------------------
  //  Make received trackback list
  //
  //  $docid     : integer : document id
  //  $tbListTpl : integer : Trackback template
  //  return     : strings : 
  //-----------------------------------------------------
  function MakeTrackbackList($docid = 0,$tbListTpl){
    global $modx,$_lang;

    if(! $rs = $modx->db->select("url, title, blog_name, excerpt, date",
				 $this->tblTrackback,
				 "     parent_id = ".$docid.
				 " and view = 1",
				 "date DESC")){
      return $_lang['fail_query'];
    }

    $i=0;
    $output = '';
    while ($row= $modx->db->getRow($rs)) {

      $i++;
      $tmptpl = $tbListTpl;
      $tmptpl = str_replace('[+tbnumber+]',$i,$tmptpl);
      $tmptpl = str_replace('[+tbtitle+]',$row['title'],$tmptpl);
      //$shortExcerpt = iconv("UTF-8","ISO-8859-1",$resource[$i]['excerpt']);
      //$tmptpl = str_replace('[+tbexcerpt+]',$shortExcerpt,$tmptpl);
      $tmptpl = str_replace('[+tbexcerpt+]',$row['excerpt'],$tmptpl);
      $tmptpl = str_replace('[+tburl+]',$row['url'],$tmptpl);
      $tmptpl = str_replace('[+tbblogname+]',$row['blog_name'],$tmptpl);
      $tmptpl = str_replace('[+tbdate+]',strftime("%Y/%m/%d %T",$row['date']),$tmptpl);

      $output .= $tmptpl;
    }
    if( $output == '' ){
      $output = $_lang['no_trackbacks'];
    }
    return $output;

  }

  //-----------------------------------------------------
  //  Recive Trackback from other blog.
  //
  //  return : string : result
  //-----------------------------------------------------
  function ReceiveTrackback(){
    global $modx;

    $errmsg = '';
    $data = array();
    $match = array();
    
    // Gather $_GET information
    if( ! isset($_GET['tb_id']) || !is_numeric($_GET['tb_id']) ) {
      $errmsg .= 'Unknown Target Document ID.'."\n";
    }else{
      $data['docid'] = $_GET['tb_id'];

      if( ! $this->CheckAllowTrackback($data['docid']) == $this->ckAllow){
	$errmsg .= 'Trackback Deny This Document ID:'.$data['docid']."\n";
      }else{

	// Get URL
	if( ! isset($_POST['url'])) {
	  $errmsg .= 'No Input URL.'."\n";
	}else{
	  $data['url'] = $this->CutText($_POST['url'],$this->StringSize);
	  $tdomain = parse_url($data['url'],PHP_URL_HOST);
	  $data['domain'] = $tdomain['host'];
	}

	// Get Title
	$data['title'] = isset($_POST['title']) ? $this->CutText($_POST['title'],$this->StringSize) : $url;
	$data['excerpt'] = isset($_POST['excerpt']) ? $this->CutText($_POST['excerpt'],$this->excerptContentSize,true) : '';
	$data['blog_name'] = isset($_POST['blog_name']) ? $this->CutText($_POST['blog_name'],$this->StringSize) : 'no name';
	
	if( isset($_SERVER['HTTP_X_FORWARDED_FOR']) && 
	    preg_match('/^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/',$_SERVER['HTTP_X_FORWARDED_FOR'],$match) != 0 ){
	  $data['remoteip'] = $match[0];
	}else{
	  $data['remoteip'] = $_SERVER['REMOTE_ADDR'];
	}
	
      }
    }

    foreach($data as $key => $val) {
      $data[$key] = $modx->stripTags($modx->db->escape($data[$key]));
    }

    if( $errmsg == '' ){
      $fields = array('id'        => '',
		      'parent_id' => $data['docid'],
		      'domain'    => $data['domain'],
		      'remoteip'  => $data['remoteip'],
		      'url'       => $data['url'],
		      'title'     => $data['title'],
		      'excerpt'   => $data['excerpt'],
		      'blog_name' => $data['blog_name'],
		      'date'      => time()
		      );
      
      if(! $modx->db->insert( $fields , $this->tblTrackback ) ){
        $errmsg .= 'Sorry,Trackback failed(system error).'."\n";      
      }
    }

    if( $errmsg != '' ){
      $this->Pnt('Failed Trackback Receive:'.$data['url']."\n\nError:".$errmsg);
      return $this->MakeResponseXML(false,$errmsg);
    }
    $this->Pnt('Successed Trackback Receive:'.$data['url']);
    return $this->MakeResponseXML(true);

  } 

  //-----------------------------------------------------
  //  Output from data in OnDocFormRender event
  //
  //  $docid    : integer : Document ID
  //  $tvPing   : integer : Trackback ping from TV
  //  $autoPing : integer : Trackback ping from Document(auto)
  //  return    : string  : formdata
  //-----------------------------------------------------
  function GetForm($docid,$tvPing,$autoPing){
    global $_lang;

    $output = '';
    if( $this->CheckAllowTrackback($docid) == $this->ckAllow){

      $tvPingCheck = $tvPing == 1 ? 'checked' : '';
      $autoPingCheck = $autoPing == 1 ? 'checked' : '';

      $output = 
	'<div class="sectionHeader">&nbsp;'.$_lang['form_title']. '</div>'."\n".
	'<div class="sectionBody">'."\n".
	'  <input type="checkbox" name="tb_enableTVPing" '.$tvPingCheck.' />&nbsp;'.$_lang['form_tvping'].'<br />'."\n".
	'  <input type="checkbox" name="tb_enableAutoPing" '.$autoPingCheck.' />&nbsp;'.$_lang['form_autoping'].'<br />'."\n".
	'</div>'."\n";
    }
    return $output;

  }

  //-----------------------------------------------------
  //  Output to MODx log
  //
  //  $msg   : string : message.
  //  $title : string : message title
  //  return : none   :
  //-----------------------------------------------------
  function Pnt($msg,$title='-'){
    global $modx;
    $modx->logEvent(1, 1, $msg, 'Trackback:'.$title);
  } 

  //-----------------------------------------------------
  //  Output to MODx log(DEBUG)
  //
  //  $msg   : string : message
  //  $title : string : message title
  //  return : none   :
  //-----------------------------------------------------
  function DPnt($msg,$title='-'){
    global $modx;    
    if( $this->DEBUG == true){
      $modx->logEvent(1, 1, $msg, '[debug]Trackback:'.$title);
    }
  } 

  //-----------------------------------------------------
  //  Set Author
  //
  //  $author : string  : default document author
  //  $force  : boolean :
  //  return  : none    :
  //-----------------------------------------------------
  function SetAuthor($author,$force=false){

    $this->author = $author;
    if( $force == true ){
      $this->forceAuthor = true;
    }else{
      $this->forceAuthor = false;
    }

  } 

  //-----------------------------------------------------
  //  Get Author
  //
  //  $userid : integer  : user id
  //  return  : string   : author name
  //-----------------------------------------------------
  function GetAuthor($userid=0){
    global $modx;

    if( $this->forceAuthor == ture ){
      return $this->author;
    }else{
      $rs = $modx->db->select("fullname", $modx->getFullTableName("user_attributes"), "id = " .$userid, "id DESC", "1");
      $resource = $modx->fetchRow($rs); 
      if( $resource['fullname'] == '' ){
	return $this->author;
      }else{
	return $resource['fullname'];
      }
    }
  }

  //-----------------------------------------------------
  //  Unix timestamp to WECDTF format
  //
  //  $time  : integer  : Unix timestamp
  //  return : string   : WECDTF or Error message
  //-----------------------------------------------------
  function TimeToW3CDTF($time){
    global $_lang;
    if( is_numeric($time)){
      return strftime("%Y-%m-%dT%T",$time);
    }
    return $_lang['invalid_timestamp'];
  }

  //-----------------------------------------------------
  //  Sending trackback ping
  //
  //  $docid : integer : document id
  //  $tbURL : string  : trackback URL
  //  return : bool    : true/false
  //-----------------------------------------------------
  function ping($docid,$tbURL){
    global $modx;
    global $_lang;

    $data  = '';
    $target= '';
    $response = '';

    $this->DPnt('Ping Process Start.','Ping - Start');

    if( $tbURL == '' || 
	! is_numeric($docid)){
      $this->Pnt('Invalid trackback URL or invalid document id','Ping');
      return false;
    }

    if(! $this->CheckAllowTrackback($docid) == $this->ckAllow){
      return false;
    }

    $docObj = $modx->getDocumentObject('id',$docid);

    $data['title'] = $docObj['pagetitle'] != '' ? $docObj['pagetitle'] : $_lang['no_pagetitle'];

    if( $docObj['introtext'] != "" ){
      $data['excerpt'] = $modx->stripTags($docObj['introtext']);
    }else{
      $data['excerpt'] = $this->CutText($modx->stripTags($docObj['content']),$this->excerptContentSize,true);
    }
    $data['url'] = $modx->makeUrl($docid,'','','full');
    $data['blog_name'] = $modx->config['site_name'];

    // Parse the target
    if( preg_match('/^https?:\/\//',$tbURL) == 0 ){
      $target = parse_url('http://'.$tbURL);
    }else{
      $target = parse_url($tbURL);
    }

    $target["query"] = (isset($target["query"])) && ($target["query"] != "") ? '?' . $target["query"]:'';

    if( ! isset($target["port"]) || ! is_numeric($target["port"])){
      $target["port"] = 80;
    } 
    // Open the socket
    $tb_sock = fsockopen($target["host"], $target["port"]); 
    // Something didn't work out, return
    if (!is_resource($tb_sock)) {
      $this->Pnt('Socket open failed.','Ping');
      return false;
      //return '$trackback->ping: can\'t connect to: ' . $tb . '.';
    }
    // Put together the things we want to send
    $tb_send = "url=" . rawurlencode($data['url']) 
      . "&title=" . rawurlencode($data['title']) 
      . "&blog_name=" . rawurlencode($data['blog_name']) 
      . "&excerpt=" . rawurlencode($data['excerpt']); 
    
    // Send the trackback
    fputs($tb_sock, "POST " . $target["path"] . $target["query"] . " HTTP/1.1\r\n");
    fputs($tb_sock, "Host: " . $target["host"] . "\r\n");
    fputs($tb_sock, "Content-type: application/x-www-form-urlencoded\r\n");
    fputs($tb_sock, "Content-length: " . strlen($tb_send) . "\r\n");
    fputs($tb_sock, "Connection: close\r\n\r\n");
    fputs($tb_sock, $tb_send); 
    // Gather result
    while (!feof($tb_sock)) {
      $response .= fgets($tb_sock, 128);
    } 
    // Close socket
    fclose($tb_sock); 

    // Did the trackback ping work
    if( strpos($response, '<error>0</error>') ){
      $this->Pnt('Trackback ping successed:'.$tbURL,'Ping');
      return true;
    }

    $this->Pnt('Trackback failed. XML Received:'.$response,'Ping');
    return false;

  } 

  //-----------------------------------------------------
  //  Search text for links, and searches links for trackback URLs.
  //
  //  $text  : string : document data
  //  return : array  : trackback URLs
  //-----------------------------------------------------
  function URLAutoDiscovery($text){ 
    // Get a list of UNIQUE links from text...
    // ---------------------------------------
    // RegExp to look for (0=>link, 4=>host in 'replace')
    $reg_exp = "/(http)+(s)?:(\\/\\/)((\\w|\\.)+)(\\/)?(\\S+)?/i"; 
    // Make sure each link ends with [sapce]
    $text = eregi_replace("www.", "http://www.", $text);
    $text = eregi_replace("http://http://", "http://", $text);
    $text = eregi_replace("\"", " \"", $text);
    $text = eregi_replace("'", " '", $text);
    $text = eregi_replace(">", " >", $text); 
    // Create an array with unique links
    $uri_array = array();
    if (preg_match_all($reg_exp, strip_tags($text, "<a>"), $array, PREG_PATTERN_ORDER)) {
      foreach($array[0] as $key => $link) {
	foreach((array(",", ".", ":", ";")) as $t_key => $t_value) {
	  $link = trim($link, $t_value);
	} 
	$uri_array[] = ($link);
      } 
      $uri_array = array_unique($uri_array);
    } 
    // Get the trackback URIs from those links...
    // ------------------------------------------
    // Loop through the URIs array and extract RDF segments
    $rdf_array = array(); // <- holds list of RDF segments
    foreach($uri_array as $key => $link) {
      if ($link_content = implode("", @file($link))) {
	preg_match_all('/(<rdf:RDF.*?<\/rdf:RDF>)/sm', $link_content, $link_rdf, PREG_SET_ORDER);
	for ($i = 0; $i < count($link_rdf); $i++) {
	  if (preg_match('|dc:identifier="' . preg_quote($link) . '"|ms', $link_rdf[$i][1])) {
	    $rdf_array[] = trim($link_rdf[$i][1]);
	  } 
	} 
      } 
    } 
    // Loop through the RDFs array and extract trackback URIs
    $tb_array = array(); // <- holds list of trackback URIs
    if (!empty($rdf_array)) {
      for ($i = 0; $i < count($rdf_array); $i++) {
	if (preg_match('/trackback:ping="([^"]+)"/', $rdf_array[$i], $array)) {
	  $tb_array[] = trim($array[1]);
	} 
      } 
    } 
    // Return Trackbacks
    return $tb_array;
  } 

  /**
   * Converts a string into an XML-safe string (replaces &, <, >, " and ')
   * 
   * @param string $string 
   * @return string 
   */
  function xml_safe($string)
  {
    return htmlspecialchars($string, ENT_QUOTES);
  } 

  //-----------------------------------------------------
  //  Response XML messsage for processing trackback.
  //
  //  $status : boolean  : Process success or fail
  //  $msg    : string   : Error messages
  //  return  : string   : XML
  //-----------------------------------------------------
  function MakeResponseXML($status = false, $msg = ""){ 
    // Default error response in case of problems...
    if (!$status && empty($msg)) {
      $msg = "Trackback failed.";
    } 
    // Start response to trackbacker...
    $output = '<?xml version="1.0" encoding="' . $this->encoding . '"?>' . "\n"
      . "<response> \n"; 
    $output .= $status == true ? "  <error>0</error>\n" : "  <error>1</error>\n  <message>" . $this->xml_safe($msg) . "</message>\n";
    $output .= "</response>";

    return $output;
  } 

  //-----------------------------------------------------
  //  Cuts a string short (with text_continue string )
  //
  //  $string : string   : text
  //  $len    : interger : max length
  //  $add    : bool     : add text_continue
  //  return  : string   : cut string
  //-----------------------------------------------------
  function CutText($string,$len=255,$add=false){
    global $_lang;

    if( $this->mbEnable == true ){
      if (mb_strlen($string) > $len) {       
	$tmpstr = mb_substr($string, 0, $len);
	return $add == true ? $tmpstr . $_lang['text_continue']:$tmpstr;
      } 
    }else{
      if (strlen($string) > $len) {
	$tmpstr = substr($string, 0, $len);
	return $add == true ? $tmpstr . $_lang['text_continue']:$tmpstr;
      } 
    }
    return $string;
  } 

  //-----------------------------------------------------
  //  Check allow trachback 
  //
  //  $docid     : integer : document ID
  //  return     : bool    : true/false
  //-----------------------------------------------------
  function CheckAllowTrackback($docid){
    global $modx;

    if( isset($allowDocID[$docid]) ){
      return $allowDocID[$docid];
    }

    $docObj = $modx->getDocumentObject('id',$docid);

    while (!empty($docObj)){

      if( isset($this->allowParentID[$docObj['parent']][0]) &&
	  ($this->allowParentID[$docObj['parent']][0] == true || $docObj['id'] == $docid) ){

	if( $this->allowParentID[$docObj['parent']][1] != 0 &&
	    time() - $docObj['createdon'] > $this->allowParentID[$docObj['parent']][1]){
	  $this->DPnt('Trackback allow is expire:'.$docid,'CheckAllowTrackback');
	  $allowDocID[$docid] = $this->ckExpire;
	  return $this->ckExpire;
	}

	$this->DPnt('Allow this document:'.$docid,'CheckAllowTrackback');
	$allowDocID[$docid] = $this->ckAllow;
	return $this->ckAllow;
      }
      
      if($docObj['parent'] == 0){
	$docObj = array();
      }else{
	$docObj = $modx->getDocumentObject('id',$docObj['parent']);
      }

    }

    $this->Pnt('Deny this document:'.$docid,'CheckAllowTrackback');
    $allowDocID[$docid] = $this->ckDeny;
    return $this->ckDeny;
  }

}
?>
